
local MAJOR_VERSION = "2.1"
local MINOR_VERSION = tonumber((string.gsub("$Revision: 0 $",
    "^.-(%d+).-$", "%1")))

local L = AceLibrary"AceLocale-2.2":new"ItsPolyquip"
local Q = AceLibrary"AceLocale-2.2":new"ItsPolyquip_Quips"
local C = AceLibrary"AceLocale-2.2":new"ItsPolyquip_CustomQuips"
local Console = AceLibrary"AceConsole-2.0"

Polyquip = AceLibrary("AceAddon-2.0")
    :new("AceConsole-2.0","AceDB-2.0","AceEvent-2.0")
local Polyquip = Polyquip

Polyquip.title = "Its' Polyquip"
Polyquip.version = MAJOR_VERSION .. "." .. MINOR_VERSION
Polyquip.date = string.gsub("$Date: 2007-02-10 00:00:00 $",
    "^.-(%d%d%d%d-%d%d-%d%d).-$", "%1")

local _G = getfenv(0)

local QUIP_DISABLED = "disabled"
local QUIP_ALWAYS = "always"
local QUIP_CHANCE = "chance"
local QUIP_ENABLED = "enabled"

local function its_in(value, ...)
    if select('#',...) == 1 and type(select(1,...)) == "table" then
        for k,v in pairs(select(1,...)) do
            if v == value then
                return true
            end
        end
        return false
    else
        for i=1,select('#',...) do
            if select(i,...) == value then
                return true
            end
        end
        return false
    end
end

local function ValidateQuipMode(x)
    local x = string.lower(x)
    return its_in(x, {"disabled","always","chance","enabled"})
end

local chatCmdTable = {
    handler = Polyquip,
    type = "group",
    args = {
        -- /polyquip chance [P]
        chance = {
            type = "range",
            name = L["Chance to quip"],
            desc = L["Chance to quip on casting Polymorph."],
            isPercent = true,
            get = "GetChance",
            set = "SetChance",
        },

        -- /polyquip channel [channel]
        channel = {
            type = "text",
            name = L["Default channel"],
            desc = L["Default channel to send quips to."],
            usage = L["<VALID_CHANNELS>"],
            validate = function(v)
                return its_in(string.upper(v), L["VALID_CHANNELS"])
            end,
            get = "GetChannel",
            set = "SetChannelUser",
        },

        -- /polyquip silence [sec]
        silence = {
            type = "text",
            name = L["Silence time"],
            desc = L["Minimum silence time between quips."],
            usage = L["<seconds>"],
            input = true,
            validate = function(v)
                local s = tonumber(v)
                return s and s >= 0
            end,
            get = "GetSilenceTime",
            set = "SetSilenceTimeUser",
        },

        -- /polyquip future
        future = {
            type = "group",
            name = L["Future features"],
            desc = L["Future (unimplemented) features."],
            args = {
                -- /polyquip future portalquip [status]
                portalquip = {
                    type = "text",
                    name = L["Portalquipping"],
                    desc = L["Portalquipping mode."],
                    usage = L["<VALID_MODES>"],
                    validate = ValidateQuipMode,
                    get = "GetPortalquipMode",
                    set = "SetPortalquipMode",
                },

                -- /polyquip future telequip [status]
                telequip = {
                    type = "text",
                    name = L["Telequipping"],
                    desc = L["Telequipping mode."],
                    usage = L["<VALID_MODES>"],
                    validate = ValidateQuipMode,
                    get = "GetTelequipMode",
                    set = "SetTelequipMode",
                },
            },
        },

        -- /polyquip when [context] [status]
        when = {
            type = "group",
            name = L["Enable when..."],
            desc = L["Contexts in which Polyquip should be enabled."],
            args = {
                -- /polyquip when solo [status]
                solo = {
                    type = "toggle",
                    name = L["Soloing"],
                    desc = L["Soloing."],
                    map = L["ENABLED_DISABLED_MAP"],
                    get = "GetSoloState",
                    set = "SetSoloState",
                },
    
                -- /polyquip when inparty [status]
                inparty = {
                    type = "toggle",
                    name = L["In party"],
                    desc = L["In a party."],
                    map = L["ENABLED_DISABLED_MAP"],
                    get = "GetPartyState",
                    set = "SetPartyState",
                },
    
                -- /polyquip when inraid [status]
                inraid = {
                    type = "toggle",
                    name = L["In raid"],
                    desc = L["In a raid group."],
                    map = L["ENABLED_DISABLED_MAP"],
                    get = "GetRaidState",
                    set = "SetRaidState",
                },
    
                -- /polyquip when inbg [status]
                inbg = {
                    type = "toggle",
                    name = L["In battleground"],
                    desc = L["In a battleground group."],
                    map = L["ENABLED_DISABLED_MAP"],
                    get = "GetBGState",
                    set = "SetBGState",
                },
            }
        }
    }
}

Polyquip:RegisterChatCommand(L["ChatCommands"], chatCmdTable)

Polyquip:RegisterDB("ItsPolyquip_DB")
Polyquip:RegisterDefaults("profile", {
    chance = 1.0,
    defaultChannel = "SAY",
    silence = 5,
    solo = true,
    inParty = true,
    inRaid = false,
    inBG = true,
    portalquip = QUIP_DISABLED,
    telequip = QUIP_DISABLED,
})

Polyquip.context = "SOLO"
Polyquip.enabled = true
Polyquip.localEnabled = true
Polyquip.lastQuip = nil

function Polyquip:OnInitialize()
end

function Polyquip:OnEnable()
    self:RegisterEvent("PARTY_MEMBERS_CHANGED")
    self:RegisterEvent("RAID_ROSTER_UPDATE")
    self:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL")
    self:RegisterEvent("UNIT_SPELLCAST_SENT")
end

function Polyquip:OnDisable()
    --[[self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
    self:UnregisterEvent("RAID_ROSTER_UPDATE")
    self:UnregisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL")
    self:UnregisterEvent("UNIT_SPELLCAST_SENT")]]
end

function Polyquip:PARTY_MEMBERS_CHANGED()
    self:UpdateContext()
end

function Polyquip:RAID_ROSTER_UPDATE()
    self:UpdateContext()
end

function Polyquip:CHAT_MSG_BG_SYSTEM_NEUTRAL()
    self:UpdateContext()
end

function Polyquip:UpdateContext()
    local bgStats = GetNumBattlefieldStats()
    
    if bgStats ~= nil and bgStats > 0 then
        self.context = "BG"
        self.localEnabled = self.db.profile.inBG
    elseif UnitInRaid("player") then
        self.context = "RAID"
        self.localEnabled = self.db.profile.inRaid
    elseif GetNumPartyMembers() ~= 0 then
        self.context = "PARTY"
        self.localEnabled = self.db.profile.inParty
    else
        self.context = "SOLO"
        self.localEnabled = self.db.profile.solo
    end
end

function Polyquip:UNIT_SPELLCAST_SENT(unit, name, rank, target)
    -- Ensure we're enabled
    --if not (self.enabled and self.localEnabled) then
    if not self.enabled then
        return
    end

    -- Make sure we're outside the silence time
    if self.lastQuip and (GetTime() - self.lastQuip)
            < self:GetSilenceTime() then
        return
    end

    local generic
    local specific
    local fmtfn

    -- Check that this is actually a Polymorph spell
    if its_in(name, L["POLYMORPH_SPELLS"]) then

        -- EDIT 2.1: Check local enabled state
        if not self.localEnabled then return end

        -- Do a chance check
        local chance = math.random()
        if chance > self.db.profile.chance then
            return
        end

        -- Get the generic and specific quip tables
        generic = self:GetGenericQuips()
        specific = nil

        if name == L["Polymorph"] then
            specific = self:GetSheepQuips()
        elseif name == L["Polymorph: Pig"] then
            specific = self:GetPigQuips()
        elseif name == L["Polymorph: Turtle"] then
            specific = self:GetTurtleQuips()
        else
            self:Print(L["Error: got an unexpected spell %s!"]
                .."  "..L["Please report this to the author as a bug."],
                name)
            return
        end

        fmtfn = function(q)
            return self:FormatQuip(q, name, target)
        end

    elseif string.find(name, L["TELEPORT_SPELLS_RE"]) then

        --[[-- Do a chance check
        local chance = math.random()
        if chance > self.db.profile.chance then
            return
        end]]

        local mode = self.db.profile.telequip
        if mode == QUIP_DISABLED then
            return
        
        elseif mode == QUIP_ALWAYS then
            -- Nothing to do
            
        elseif mode == QUIP_CHANCE then
            if math.random() > self.db.profile.chance then
                return
            end

        elseif mode == QUIP_ENABLED then
            if not self.localEnabled then return end
            --[[if math.random() > self.db.profile.chance then
                return
            end]]

        end

        -- Get the generic and specific quip tables
        generic = self:GetGenericTeleportQuips()
        local destre = L["TELEPORT_DEST_RE"]
        local dest = string.gsub(name, unpack(destre))
        specific = self:GetSpecificTeleportQuips(dest)

        fmtfn = function(q)
            return self:FormatPortalQuip(q, dest)
        end

    elseif string.find(name, L["PORTAL_SPELLS_RE"]) then

        --[[-- Skip the chance check]]

        local mode = self.db.profile.portalquip
        if mode == QUIP_DISABLED then
            return
        
        elseif mode == QUIP_ALWAYS then
            -- Nothing to do
            
        elseif mode == QUIP_CHANCE then
            if math.random() > self.db.profile.chance then
                return
            end

        elseif mode == QUIP_ENABLED then
            if not self.localEnabled then return end
            --[[if math.random() > self.db.profile.chance then
                return
            end]]

        end

        -- Get the generic and specific quip tables
        generic = self:GetGenericPortalQuips()
        local destre = L["PORTAL_DEST_RE"]
        local dest = string.gsub(name, unpack(destre))
        specific = self:GetSpecificPortalQuips(dest)

        fmtfn = function(q)
            return self:FormatPortalQuip(q, dest)
        end
        
    else
        return
    end

    -- Break up the generic and specific sets
    local geneBase, geneCust, specBase, specCust
    geneBase = generic[1]
    geneCust = generic[2]
    specBase = specific[1]
    specCust = specific[2]

    -- Pick a quip from the "combined" tables.
    local index = math.random(#geneBase+#geneCust+#specBase+#specCust)
    local quip = nil

    if index <= #geneBase and #geneBase ~= 0 then
        if self.debug then self:Print("Generic base quip.") end
        quip = geneBase[index]
    elseif index <= #geneBase+#geneCust and #geneCust ~= 0 then
        if self.debug then self:Print("Generic custom quip.") end
        quip = geneCust[index-#geneBase]
    elseif index <= #geneBase+#geneCust+#specBase and #specBase ~= 0 then
        if self.debug then self:Print("Specific base quip.") end
        quip = specBase[index-#geneBase-#geneCust]
    else
        if self.debug then self:Print("Specific custom quip.") end
        quip = specCust[index-#geneBase-#geneCust-#specBase]
    end

    if self.debug then 
    self:Print("Selected quip %s from %s (%s+%s+%s+%s) total.", index,
        #geneBase+#geneCust+#specBase+#specCust,
        #geneBase,#geneCust,#specBase,#specCust)
    end

    if quip[2] == nil then
        self:Print(L["ERR_NILQUIP"])
        for _,line in L["ERR_NILQUIP_INSTRUCTIONS"] do
            Console:Print(line)
        end
        return
    end

    -- Do the quip
    local channel = quip[1]
    --local text = self:FormatQuip(quip[2], name, target)
    local text = fmtfn(quip[2])

    if channel == "DEFAULT" then
        channel = self.db.profile.defaultChannel
    end

    if channel == "EMOTE" then
        DoEmote(text)
    else
        if channel == "GROUP" then
            if self.context == "BG" then
                channel = "BATTLEGROUND"
            elseif self.context == "RAID" then
                channel = "RAID"
            elseif self.context == "PARTY" then
                channel = "PARTY"
            else
                channel = "SAY"
            end
        elseif channel == "YELL" then
            local posX, posY = GetPlayerMapPosition("player")
            -- If the player is outside, don't yell to avoid annoying
            -- everyone else in the zone...
            if posX ~= 0 or posY ~= 0 then
                channel = "SAY"
            end
        end

        SendChatMessage(text, channel)
    end

    -- Update silence timestamp
    self.lastQuip = GetTime()
end

function Polyquip:FormatQuip(text, spell, targetName)
    local subs = {}
    local copySub = nil

    subs.t = targetName

    if spell == L["Polymorph"] then
        copySub = L["Substitutions"].sheep
    elseif spell == L["Polymorph: Pig"] then
        copySub = L["Substitutions"].pig
    elseif spell == L["Polymorph: Turtle"] then
        copySub = L["Substitutions"].turtle
    end

    for i,v in pairs(copySub) do
        subs[i] = copySub[i]
    end

    if not subs.F then subs.F = subs.A end
    if not subs.f then subs.f = subs.a end
    if not subs.Fi then subs.Fi = subs.Ai end
    if not subs.fi then subs.fi = subs.ai end

    local function sub(pat, repl)
        if pat and repl then
            return string.gsub(text, "%%"..pat, repl)
        else
            return text
        end
    end

    if subs.Ai then text = sub("Ai", subs.Ai) end
    if subs.ai then text = sub("ai", subs.ai) end
    if subs.Fi then text = sub("Fi", subs.Fi) end
    if subs.fi then text = sub("fi", subs.fi) end

    text = sub("A", subs.A)
    text = sub("a", subs.a)
    text = sub("F", subs.F)
    text = sub("f", subs.f)
    text = sub("P", subs.P)
    text = sub("p", subs.p)
    text = sub("T", subs.t)
    text = sub("t", subs.t)

    return text
end

function Polyquip:FormatPortalQuip(text, destination)
    local subs = {}
    local copySub = nil

    if UnitFactionGroup("player") == "Alliance" then
        copySub = L["Substitutions"].portals.alliance
    else
        copySub = L["Substitutions"].portals.horde
    end

    for i,v in pairs(copySub) do
        subs[i] = copySub[i]
    end

    subs.t = destination
    subs.n = UnitName('player')

    local function sub(pat, repl)
        if pat and repl then
            return string.gsub(text, "%%"..pat, repl)
        else
            return text
        end
    end

    text = sub("1", subs[1])
    text = sub("2", subs[2])
    text = sub("3", subs[3])
    text = sub("4", subs[4])
    text = sub("H", subs.h)
    text = sub("h", subs.h)
    text = sub("J", subs.j)
    text = sub("j", subs.j)
    text = sub("N", subs.n)
    text = sub("n", subs.n)
    text = sub("T", subs.t)
    text = sub("t", subs.t)
    text = sub("X", subs.x)
    text = sub("x", subs.x)

    return text
end

local function gettr(name)
    local base, custom

    if Q:HasTranslation(name) then
        base = Q[name]
    else
        base = {}
    end

    local succ,result = pcall(function()return C:HasTranslation(name)end)

    if succ and result == true then
        custom = C[name]
    else
        custom = {}
    end

    return {base, custom}
end

function Polyquip:GetGenericQuips()
    return gettr("POLY_GENERIC")
end

function Polyquip:GetSheepQuips()
    return gettr("POLY_SHEEP")
end

function Polyquip:GetPigQuips()
    return gettr("POLY_PIG")
end

function Polyquip:GetTurtleQuips()
    return gettr("POLY_TURTLE")
end

function Polyquip:GetGenericTeleportQuips()
    return gettr("TELEPORT_GENERIC")
end

function Polyquip:GetSpecificTeleportQuips(destination)
    local key = "TELEPORT_" .. string.upper(destination)
    return gettr(key)
end

function Polyquip:GetGenericPortalQuips()
    local key = "PORTAL_GENERIC"
    return gettr(key)
end

function Polyquip:GetSpecificPortalQuips(destination)
    local key = "PORTAL_" .. string.upper(destination)
    return gettr(key)
end

function Polyquip:GetChance()
    return self.db.profile.chance
end

function Polyquip:SetChance(P)
    self.db.profile.chance = P
end

function Polyquip:GetChannel()
    return self.db.profile.defaultChannel
end

function Polyquip:GetChannelUser()
    return L["CHANNEL_"..self:GetChannel()]
end

function Polyquip:SetChannel(channel)
    self.db.profile.defaultChannel = channel
end

function Polyquip:SetChannelUser(channel)
    local lc = string.lower
    local channel = lc(channel)
    
    if channel == lc(L["CHANNEL_SAY"]) then
        self:SetChannel("SAY")
    elseif channel == lc(L["CHANNEL_GROUP"]) then
        self:SetChannel("GROUP")
    end
end

function Polyquip:GetSilenceTime()
    return self.db.profile.silence
end

function Polyquip:SetSilenceTime(secs)
    self.db.profile.silence = secs
end

function Polyquip:SetSilenceTimeUser(secs)
    local s = tonumber(secs)
    if s then self:SetSilenceTime(s) end
end

function Polyquip:GetSoloState()
    return self.db.profile.solo
end

function Polyquip:SetSoloState(flag)
    self.db.profile.solo = flag
end

function Polyquip:GetPartyState()
    return self.db.profile.inParty
end

function Polyquip:SetPartyState(flag)
    self.db.profile.inParty = flag
end

function Polyquip:GetRaidState()
    return self.db.profile.inRaid
end

function Polyquip:SetRaidState(flag)
    self.db.profile.inRaid = flag
end

function Polyquip:GetBGState()
    return self.db.profile.inBG
end

function Polyquip:SetBGState(flag)
    self.db.profile.inBG = flag
end

--[[function Polyquip:GetPortalsEnabled()
    return self.db.profile.portals
end

function Polyquip:SetPortalsEnabled(flag)
    self.db.profile.portals = true
end]]

function Polyquip:GetPortalquipMode()
    return self.db.profile.portalquip
end

function Polyquip:SetPortalquipMode(value)
    self.db.profile.portalquip = string.lower(value)
end

function Polyquip:GetTelequipMode()
    return self.db.profile.telequip
end

function Polyquip:SetTelequipMode(value)
    self.db.profile.telequip = string.lower(value)
end

