--[[
    
Adding custom quips is fairly simple.  You merely need to rename this file to 
"Quips-custom.lua", and then start adding quips.

Each quip needs to be placed in the appropriate section.  These are:

    POLY_GENERIC    -- quips used for all Polymorph spells,
    POLY_SHEEP      -- quips for regular Polymorph,
    POLY_PIG        -- quips used for Polymorph: Pig and
    POLY_TURTLE     -- quips used for Polymorph: Turtle.

Each individual quip has two parts: the channel it should be broadcast on,
and the content of the quip.  For your convenience, there are several
functions defined below that you can use for the various channels.  These
are:

    DEF     -- quip is sent on the default channel set by the user,
    SAY     -- quip is said out loud,
    PARTY   -- quip is sent to the party channel,
    RAID    -- quip is sent to the raid channel,
    YELL    -- quip is yelled,
    GROUP   -- quip is said to the raid, party or out loud
               (they are tried in that order) and
    EMOTE   -- the quip is "said" as an emote.

For most quips, just use "DEF", although "EMOTE" is useful for special
emotes.  Also note that unless you are in an instance, the "YELL" channel
will actually be only said out loud--this is to avoid annoying everyone
in the zone.

Finally, each quip can use various text substitutions that expand to
different values depending on the spell, target, etc.  Here is a full list:

   Code Purpose             Example
    %A  Animal name         Pig
    %F  Alternate name      Pork
    %P  Spell short name    Piggymorph
    %T  Target's name       Scarlet Zealot

A few notes on the above: for all but the target's name, these
substitutions will honour case; that is, for Polymorph: Pig, "%A" becomes
"Pig" and "%a" becomes "pig".  Also, they will correctly compensate for
trailing 'i's; for Polymorph: Turtle, "%Aing" becomes "Turtling" as
opposed to the incorrect "Turtleing".

Now, for an example.  Here, we will add a new, witty quip for when we use
any variant of Polymorph (just copy and paste the below code into the
marked section at the bottom of this file).  It will be broadcast over the
user's default channel.

    ["POLY_GENERIC"] = {
        DEF"You are in my power, %t; now you shall become a chicken!  ...  Oh damnit, this NEVER works right!",
    },

Here it is again, over multiple lines, along with a second quip.  Remember,
you cannot have multiple sections of the same name!

    ["POLY_GENERIC"] = {
        DEF("You are in my power, %t; now you shall become a chicken!"
            .. "  ...  Oh damnit, this NEVER works right!"),

        DEF"Mmm... you look tasty, %t... let me make one minor change...",
    },

Have fun, and remember to send in your custom quips so other people can
use them!

]]

local function DEF(x) return {"DEFAULT", x} end
local function SAY(x) return {"SAY", x} end
local function PARTY(x) return {"PARTY", x} end
local function RAID(x) return {"RAID", x} end
local function YELL(x) return {"YELL", x} end
local function GROUP(x) return {"GROUP", x} end
local function EMOTE(x) return {"EMOTE", x} end

AceLibrary("AceLocale-2.2"):new("ItsPolyquip_CustomQuips")
:RegisterTranslations("enUS", function() return
{
    --[[ Add your custom quips below here ]]--

    --[[ Add your custom quips above here ]]--
}
end)
